==> redis-1.2.0.tar.gz: lib/cas.dart <==

/*
 * Free software licenced under 
 * GNU AFFERO GENERAL PUBLIC LICENSE
 * 
 * Check for document LICENCE forfull licence text
 * 
 * Luka Rahne
 */

part of redis;

class Cas{
  Command _cmd;
  Completer<bool> _completer_bool;
  
  /// Class for  CAS  check-and-set pattern  
  /// Construct with Command and call  
  /// watch() and multiAndExec()  
  Cas(this._cmd){}
  
  /// watch takes list of watched keys and  
  /// function that is executed  
  /// during CAS opertion  
  /// 
  Future watch(List<String> watching_keys,  func()){
     //return _cmd.send_object(["TRANS"]);
    List<String> watchcmd = ["WATCH"];
    watchcmd.addAll(watching_keys);
    return Future.doWhile((){
      _completer_bool = new Completer();
      _cmd.send_object(watchcmd).then((_){
        func();
      });
      return _completer_bool.future;
    });
  }
  
  /// multiAndExec takes function 
  /// to complete CAS as Transaction
  /// passed function takes Transation
  /// as only parameter and should be used to 
  /// complete transaction
  /// 
  /// !!! DO NOT call exec() on Transation
  
  Future multiAndExec(Future func(Transation)){
    return _cmd.multi().then((Transaction _trans){
      func(_trans);
      return _trans.exec().then((var resp){
        if (resp == "OK"){
          _completer_bool.complete(false); //terminate Future.doWhile
        }
        else{
          _completer_bool.complete(true);
        }
      });
    });
  }
}


class M {
  bool _val;
  M(this._val);
}

class W {
  bool _val;
  W(this._val);
}

class E {
  bool _val;
  E(this._val);
}

class CAS{
  Command _cmd;
  Transaction _trans;
  /// Class for  CAS  check-and-set pattern  
  /// Construct with Command and call  
  /// watch() and multiAndExec()  
  CAS(this._cmd){}
  
  Future watch(List<String> watching_keys, Future<M> f()){
     //return _cmd.send_object(["TRANS"]);
    List<String> watchcmd = ["WATCH"];
    watchcmd.addAll(watching_keys);
    return Future.doWhile((){
      return _cmd.send_object(watchcmd).then((_){
        return f().then<bool>((v)=>v._val);
      });
    });
  }
  
  Future<M> multi(Future<E> Function(Transaction) func){
    return _cmd.multi().then((Transaction t){
          this._trans = t;
          return func(t);
    }).then<M>((E e){ 
      return M(e._val); 
    });
  }

  Future<E> exec(){
    return _trans.exec().then<E>((var resp){
      bool r = resp == "OK" ? false : true;
      return E(r); // doWhile is terminated on false;
    });
  }
}

==> redis-1.4.0.tar.gz: lib/cas.dart <==

/*
 * Free software licenced under 
 * GNU AFFERO GENERAL PUBLIC LICENSE
 * 
 * Check for document LICENCE forfull licence text
 * 
 * Luka Rahne
 */

part of redis;

class Cas{
  Command _cmd;
  Completer<bool> _completer_bool;
  
  /// Class for  CAS  check-and-set pattern  
  /// Construct with Command and call  
  /// watch() and multiAndExec()  
  Cas(this._cmd){}
  
  /// watch takes list of watched keys and  
  /// function that is executed  
  /// during CAS opertion  
  /// 
  Future watch(List<String> watching_keys,  func()){
     //return _cmd.send_object(["TRANS"]);
    List<String> watchcmd = ["WATCH"];
    watchcmd.addAll(watching_keys);
    return Future.doWhile((){
      _completer_bool = new Completer();
      _cmd.send_object(watchcmd).then((_){
        func();
      });
      return _completer_bool.future;
    });
  }
  
  /// multiAndExec takes function 
  /// to complete CAS as Transaction
  /// passed function takes Transation
  /// as only parameter and should be used to 
  /// complete transaction
  /// 
  /// !!! DO NOT call exec() on Transation
  
  Future multiAndExec(Future func(Transation)){
    return _cmd.multi().then((Transaction _trans){
      func(_trans);
      return _trans.exec().then((var resp){
        if (resp == "OK"){
          _completer_bool.complete(false); //terminate Future.doWhile
        }
        else{
          // exec completes only with valid response
          _completer_bool.completeError(RedisError("exec response is not expceted, but is $resp"));
        }
      })
      .catchError((e){
          // dont do anything
         _completer_bool.complete(true); // retry
       }, test: (e) => e is TransactionError);
    });
  }
}
